/*
 * wxodbc.cc
 * wxWindows ODBC commands (Windows only)
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#if defined(wx_msw) && USE_ODBC && WXODBC

#include "wx_hash.h"
#include "wx_types.h"
#include "wxstring.h"
#include "wx_db.h"

#include "wxextend.h"

char *wxSqlRetCodeToString(wxRETCODE retCode);
int wxSqlOptionsFromString(char *opt);
char *wxSqlTypeToString(long typ);

/*
 * Database
 */
 
class wxExtensionDatabase: public wxDatabase
{
 public:
  wxExtensionDatabase(void)
  {
    wxStoreThingInTable((long)this, this, wxTYPE_DATABASE);
  }
  ~wxExtensionDatabase(void)
  {
    wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_DATABASE, "OnDelete");
    if (func)
      (* (void (*)(long)) func) ((long)this);
    wxDeleteThingInTable((long)this);
  }
};

/*
 * Record set
 */
 
class wxExtensionRecordSet: public wxRecordSet
{
 public:
  wxExtensionRecordSet(wxDatabase *parent = NULL, int openType = wxOPEN_TYPE_DYNASET, int options = wxOPTION_DEFAULT):
    wxRecordSet(parent, openType, options)
  {
    wxStoreThingInTable((long)this, this, wxTYPE_RECORDSET);
  }
  ~wxExtensionRecordSet(void)
  {
    wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_RECORDSET, "OnDelete");
    if (func)
      (* (void (*)(long)) func) ((long)this);
    wxDeleteThingInTable((long)this);
  }
};

// Create database
WXID wxcDatabaseCreate(void)
{
  return (WXID) new wxExtensionDatabase();
}

// Open database
Bool wxcDatabaseOpen(WXID id, char *source, Bool exclusive, Bool readOnly, char *username, char *password)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(id, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in database-open: could not find database " << id << ".\n";
    return FALSE;
  }
  return db->Open(source, exclusive, readOnly, username, password);
}

// Close database
Bool wxcDatabaseClose(WXID id)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(id, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in database-close: could not find database " << id << ".\n";
    return FALSE;
  }
  return db->Close();
}

// Is database open?
Bool wxcDatabaseIsOpen(WXID id)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(id, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in database-is-open: could not find database " << id << ".\n";
    return FALSE;
  }
  return db->IsOpen();
}

// Return error code
long wxcDatabaseGetErrorCode(WXID id)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(id, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in database-get-error-code: could not find database " << id << ".\n";
    return -3;
  }
  return (long)db->GetErrorCode();
}

// Return error code
char *wxcDatabaseGetErrorCodeString(WXID id)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(id, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in database-get-error-code-string: could not find database " << id << ".\n";
    return "SQL_UNKNOWN";
  }
  wxRETCODE retCode = db->GetErrorCode();
  return wxSqlRetCodeToString(retCode);
}

// Return database name
char *wxcDatabaseGetDatabaseName(WXID id)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(id, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in database-get-database-name: could not find database " << id << ".\n";
    return NULL;
  }
  return db->GetDatabaseName();
}

Bool wxcDatabaseErrorOccurred(WXID id)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(id, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in database-error-occurred: could not find database " << id << ".\n";
    return TRUE;
  }
  // Note wrong spelling!
  return db->ErrorOccured();
}

char *wxcDatabaseGetErrorMessage(WXID id)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(id, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in database-get-error-message: could not find database " << id << ".\n";
    return NULL;
  }
  return db->GetErrorMessage();
}

long wxcDatabaseGetErrorNumber(WXID id)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(id, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in database-get-error-number: could not find database " << id << ".\n";
    return 0;
  }
  return db->GetErrorNumber();
}

// Delete database
Bool wxcDatabaseDelete(WXID id)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(id, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in database-delete: could not find database " << id << ".\n";
    return FALSE;
  }
  delete db;
  return TRUE;
}

/*
 * Record set functions
 */

// Create recordset
WXID wxcRecordSetCreate(WXID dbId, int openType, int options)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(dbId, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in record-set-create: could not find database " << dbId << ".\n";
    return 0;
  }
  return (WXID) new wxExtensionRecordSet(db, openType, options);
}
 
WXID wxcRecordSetCreate(WXID dbId, char *openType, char *options)
{
  wxDatabase *db = (wxDatabase *)wxGetTypedObject(dbId, wxTYPE_DATABASE);
  if (!db)
  {
    ExtensionError << "Error in record-set-create: could not find database " << dbId << ".\n";
    return 0;
  }
  if (!openType)
    openType = "wxOPEN_TYPE_DYNASET";
  if (!options)
    options = "wxOPTION_DEFAULT";
    
  int openTypeInt = wxSqlOptionsFromString(openType);
  int optionsInt = wxSqlOptionsFromString(options);
  return (WXID) new wxExtensionRecordSet(db, openTypeInt, optionsInt);
}

Bool wxcRecordSetDelete(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-create: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  delete rs;
  return TRUE;
}

Bool wxcRecordSetQuery(WXID rsId, char *cols, char *table, char *filter)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-query: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->Query(cols, table, filter);
}

int wxcRecordSetGetNumberFields(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-number-fields: could not find recordset " << rsId << ".\n";
    return 0;
  }

  return rs->GetNumberFields();
}

int wxcRecordSetGetNumberParams(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-number-params: could not find recordset " << rsId << ".\n";
    return 0;
  }

  return rs->GetNumberParams();
}

long wxcRecordSetGetNumberRecords(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-number-records: could not find recordset " << rsId << ".\n";
    return 0;
  }

  return rs->GetNumberRecords();
}

long wxcRecordSetGetNumberCols(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-number-cols: could not find recordset " << rsId << ".\n";
    return 0;
  }

  return rs->GetNumberCols();
}

char *wxcRecordSetGetFilter(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-filter: could not find recordset " << rsId << ".\n";
    return NULL;
  }

  return rs->GetFilter();
}

char *wxcRecordSetGetSortString(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-sort-string: could not find recordset " << rsId << ".\n";
    return NULL;
  }

  return rs->GetSortString();
}

WXID wxcRecordSetGetDatabase(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-database: could not find recordset " << rsId << ".\n";
    return NULL;
  }

  return (WXID)rs->GetDatabase();
}

long wxcRecordSetGetErrorCode(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-error-code: could not find recordset " << rsId << ".\n";
    return NULL;
  }

  return (long)rs->GetErrorCode();
}

char *wxcRecordSetGetErrorCodeString(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-error-code-string: could not find recordset " << rsId << ".\n";
    return NULL;
  }

  wxRETCODE retCode =  rs->GetErrorCode();
  return wxSqlRetCodeToString(retCode);
}

Bool wxcRecordSetGetResultSet(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-result-set: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->GetResultSet();
}

Bool wxcRecordSetExecuteSql(WXID rsId, char *sql)
{
  if (!sql)
    return FALSE;
    
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-execute-sql: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->ExecuteSQL(sql);
}

Bool wxcRecordSetGetTables(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-tables: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->GetTables();
}

Bool wxcRecordSetGetColumns(WXID rsId, char *table)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-columns: could not find recordset " << rsId << ".\n";
    return FALSE;
  }
  if (table && strcmp(table, "") == 0)
    table = NULL;
    
  return rs->GetColumns(table);
}

Bool wxcRecordSetGetPrimaryKeys(WXID rsId, char *table)
{
#if wxRELEASE_NUMBER > 3
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-primary-keys: could not find recordset " << rsId << ".\n";
    return FALSE;
  }
  if (table && strcmp(table, "") == 0)
    table = NULL;
    
  return rs->GetPrimaryKeys(table);
#else
  return FALSE;
#endif
}

Bool wxcRecordSetGetForeignKeys(WXID rsId, char *table, char *fName)
{
#if wxRELEASE_NUMBER > 3
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-foreign-keys: could not find recordset " << rsId << ".\n";
    return FALSE;
  }
  if (table && strcmp(table, "") == 0)
    table = NULL;
  if (fName && strcmp(fName, "") == 0)
    fName = NULL;
    
  return rs->GetForeignKeys(table, fName);
#else
  return FALSE;
#endif
}

char *wxcRecordSetGetTableName(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-table-name: could not find recordset " << rsId << ".\n";
    return NULL;
  }

  return rs->GetTableName();
}

Bool wxcRecordSetSetTableName(WXID rsId, char *table)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-set-table-name: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  rs->SetTableName(table);
  return TRUE;
}

Bool wxcRecordSetIsOpen(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-is-open: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->IsOpen();
}

Bool wxcRecordSetIsBOF(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-is-bof: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->IsBOF();
}

Bool wxcRecordSetIsEOF(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-is-eof: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->IsEOF();
}

Bool wxcRecordSetIsDeleted(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-is-deleted: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->IsDeleted();
}

long wxcRecordSetGetIntData(WXID rsId, int colPos)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-int-data: could not find recordset " << rsId << ".\n";
    return 0;
  }

  switch (rs->GetColType(colPos))
  {
    case SQL_FLOAT:
      return (long) *((double *)rs->GetFieldDataPtr(colPos, SQL_FLOAT));
    case SQL_DOUBLE:
      return (long) *((double *)rs->GetFieldDataPtr(colPos, SQL_DOUBLE));
    case SQL_INTEGER:
      return *(long *)rs->GetFieldDataPtr(colPos, SQL_INTEGER);
    case SQL_SMALLINT:
      return (long) *(short *)rs->GetFieldDataPtr(colPos, SQL_SMALLINT);
    case SQL_REAL:
      return (long) *(float *)rs->GetFieldDataPtr(colPos, SQL_REAL);
    default:
      return 0;
  }
  return 0;
}

long wxcRecordSetGetIntData(WXID rsId, char *col)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-int-data: could not find recordset " << rsId << ".\n";
    return 0;
  }

  switch (rs->GetColType(col))
  {
    case SQL_FLOAT:
      return (long) *((double *)rs->GetFieldDataPtr(col, SQL_FLOAT));
    case SQL_DOUBLE:
      return (long) *((double *)rs->GetFieldDataPtr(col, SQL_DOUBLE));
    case SQL_INTEGER:
      return *(long *)rs->GetFieldDataPtr(col, SQL_INTEGER);
    case SQL_SMALLINT:
      return (long) *(short *)rs->GetFieldDataPtr(col, SQL_SMALLINT);
    case SQL_REAL:
      return (long) *(float *)rs->GetFieldDataPtr(col, SQL_REAL);
    default:
      return 0;
  }

  return 0;
}

double wxcRecordSetGetFloatData(WXID rsId, int colPos)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-float-data: could not find recordset " << rsId << ".\n";
    return 0;
  }

  switch (rs->GetColType(colPos))
  {
    case SQL_FLOAT:
      return *((double *)rs->GetFieldDataPtr(colPos, SQL_FLOAT));
    case SQL_DOUBLE:
      return *((double *)rs->GetFieldDataPtr(colPos, SQL_DOUBLE));
    case SQL_INTEGER:
      return (double) *(long *)rs->GetFieldDataPtr(colPos, SQL_INTEGER);
    case SQL_SMALLINT:
      return (double) *(short *)rs->GetFieldDataPtr(colPos, SQL_SMALLINT);
    case SQL_REAL:
      return (double) *(float *)rs->GetFieldDataPtr(colPos, SQL_REAL);
    default:
      return 0.0;
  }
  
  return 0.0;
}

double wxcRecordSetGetFloatData(WXID rsId, char *col)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-float-data: could not find recordset " << rsId << ".\n";
    return 0;
  }

  switch (rs->GetColType(col))
  {
    case SQL_FLOAT:
      return *((double *)rs->GetFieldDataPtr(col, SQL_FLOAT));
    case SQL_DOUBLE:
      return *((double *)rs->GetFieldDataPtr(col, SQL_DOUBLE));
    case SQL_INTEGER:
      return (double) *(long *)rs->GetFieldDataPtr(col, SQL_INTEGER);
    case SQL_SMALLINT:
      return (double) *(short *)rs->GetFieldDataPtr(col, SQL_SMALLINT);
    case SQL_REAL:
      return (double) *(float *)rs->GetFieldDataPtr(col, SQL_REAL);
    default:
      return 0.0;
  }
  return 0.0;
}

char *wxcRecordSetGetCharData(WXID rsId, int colPos)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-char-data-pos: could not find recordset " << rsId << ".\n";
    return NULL;
  }

  switch (rs->GetColType(colPos))
  {
    case SQL_CHAR:
      return (char *)rs->GetFieldDataPtr(colPos, SQL_CHAR);
    case SQL_VARCHAR:
      return (char *)rs->GetFieldDataPtr(colPos, SQL_VARCHAR);
    default:
      return NULL;
  }
  return NULL;
}

char *wxcRecordSetGetCharData(WXID rsId, char *col)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-char-data: could not find recordset " << rsId << ".\n";
    return NULL;
  }

  switch (rs->GetColType(col))
  {
    case SQL_CHAR:
      return (char *)rs->GetFieldDataPtr(col, SQL_CHAR);
    case SQL_VARCHAR:
      return (char *)rs->GetFieldDataPtr(col, SQL_VARCHAR);
    default:
      return NULL;
  }
  return NULL;
}

char *wxcRecordSetGetColName(WXID rsId, int pos)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-col-name: could not find recordset " << rsId << ".\n";
    return NULL;
  }

  return rs->GetColName(pos);
}

long wxcRecordSetGetColType(WXID rsId, int pos)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-col-type: could not find recordset " << rsId << ".\n";
    return 0;
  }

  return rs->GetColType(pos);
}

long wxcRecordSetGetColType(WXID rsId, char *col)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-col-type: could not find recordset " << rsId << ".\n";
    return 0;
  }

  return rs->GetColType((const char *)col);
}

char *wxcRecordSetGetColTypeString(WXID rsId, int pos)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-col-type-string: could not find recordset " << rsId << ".\n";
    return NULL;
  }

  return wxSqlTypeToString(rs->GetColType(pos));
}

char *wxcRecordSetGetColTypeString(WXID rsId, char *col)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-col-type-string: could not find recordset " << rsId << ".\n";
    return NULL;
  }

  return wxSqlTypeToString(rs->GetColType((const char *)col));
}

Bool wxcRecordSetMove(WXID rsId, long rows)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-move: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->Move(rows);
}

Bool wxcRecordSetMoveFirst(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-move-first: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->MoveFirst();
}

Bool wxcRecordSetMoveLast(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-move-last: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->MoveLast();
}

Bool wxcRecordSetMoveNext(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-move-next: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->MoveNext();
}

Bool wxcRecordSetMovePrev(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-move-prev: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->MovePrev();
}

Bool wxcRecordSetGoto(WXID rsId, long row)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-goto: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->GoTo(row);
}

Bool wxcRecordSetGetDataSources(WXID rsId)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-get-data-sources: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->GetDataSources();
}

Bool wxcRecordSetIsFieldDirty(WXID rsId, int col)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-is-field-dirty: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->IsFieldDirty(col);
}

Bool wxcRecordSetIsFieldDirty(WXID rsId, char *col)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-is-field-dirty: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->IsFieldDirty(col);
}

Bool wxcRecordSetIsFieldNull(WXID rsId, int col)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-is-field-null: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->IsFieldNull(col);
}

Bool wxcRecordSetIsFieldNull(WXID rsId, char *col)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-is-field-null: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->IsFieldNull((const char *)col);
}

Bool wxcRecordSetIsColNullable(WXID rsId, int col)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-is-col-nullable: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->IsColNullable(col);
}

Bool wxcRecordSetIsColNullable(WXID rsId, char *col)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-is-col-nullable: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  return rs->IsColNullable((const char *)col);
}

Bool wxcRecordSetSetFieldDirty(WXID rsId, int col, Bool dirty)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-set-field-dirty: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  rs->SetFieldDirty(col, dirty);
  return TRUE;
}

Bool wxcRecordSetSetFieldDirty(WXID rsId, char *col, Bool dirty)
{
  wxRecordSet *rs = (wxRecordSet *)wxGetTypedObject(rsId, wxTYPE_RECORDSET);
  if (!rs)
  {
    ExtensionError << "Error in record-set-set-field-dirty: could not find recordset " << rsId << ".\n";
    return FALSE;
  }

  rs->SetFieldDirty((const char *)col, dirty);
  return TRUE;
}


/*
 * Utilities
 */

char *wxSqlRetCodeToString(wxRETCODE retCode)
{
  switch (retCode)
  {
    case SQL_ERROR:
      return "SQL_ERROR";
    case SQL_INVALID_HANDLE:
      return "SQL_INVALID_HANDLE";
    case SQL_NEED_DATA:
      return "SQL_NEED_DATA";
    case SQL_NO_DATA_FOUND:
      return "SQL_NO_DATA_FOUND";
    case SQL_SUCCESS:
      return "SQL_SUCCESS";
    case SQL_SUCCESS_WITH_INFO:
      return "SQL_SUCCESS_WITH_INFO";
    default:
      return "SQL_UNKNOWN";
  }
  return "SQL_UNKNOWN";
}

int wxSqlOptionsFromString(char *opt)
{
  if (!opt)
    return 0;
  wxString str(opt);
  if (str == "wxOPEN_TYPE_DYNASET")
    return wxOPEN_TYPE_DYNASET;
  else if (str == "wxOPEN_TYPE_SNAPSHOT")
    return wxOPEN_TYPE_SNAPSHOT;
  else if (str == "wxOPEN_TYPE_FORWARD_ONLY")
    return wxOPEN_TYPE_FORWARD_ONLY;
  else if (str == "wxOPTION_DEFAULT")
    return wxOPTION_DEFAULT;
  else if (str == "wxOPTION_APPEND_ONLY")
    return wxOPTION_APPEND_ONLY;
  else if (str == "wxOPTION_READ_ONLY")
    return wxOPTION_READ_ONLY;
  return 0;
}

char *wxSqlTypeToString(long typ)
{
  switch (typ)
  {
    case SQL_CHAR:
      return "SQL_CHAR";
    case SQL_VARCHAR:
      return "SQL_VARCHAR";
    case SQL_INTEGER:
      return "SQL_INTEGER";
    case SQL_FLOAT:
      return "SQL_FLOAT";
    case SQL_DOUBLE:
      return "SQL_DOUBLE";
    case SQL_SMALLINT:
      return "SQL_SMALLINT";
    case SQL_REAL:
      return "SQL_REAL";
    case SQL_TYPE_NULL:
      return "SQL_TYPE_NULL";
    default:
      return "SQL_UNKNOWN";
  }

  return "SQL_UNKNOWN";
}

#endif
